# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gtk, gconf, time
import Conf, func

class Client :

	CLIENT = None

	def __init__ (self) :

		self.CLIENT = gconf.client_get_default();

	def add_dir (self, path) :
		self.CLIENT.add_dir(os.path.dirname(path), gconf.CLIENT_PRELOAD_NONE)

	def set_path (self, path, make_new=False) :
		path = os.path.dirname(path)

		if not self.CLIENT.dir_exists(path) :
			if make_new :
				return self.create_new(path)
			else :
				raise ValueError, "%s does not exists." % path

		else :
			return True

	def get_value_from_schema_by_conf (self, conf_key) :
		if not Conf.SCHEMA.has_key(conf_key) : return None

		__value = self.CLIENT.get_default_from_schema(Conf.SCHEMA[conf_key]["key"])

		return self.get_value_from_value(__value)

	def get_value_by_conf (self, conf_key) :
		if not Conf.SCHEMA.has_key(conf_key) :
			return None

		return self.get_value(Conf.SCHEMA[conf_key]["key"])

	def get_value (self, key, typeValue=None) :

		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			return None

		if typeValue == "int_list" :
			return self.get_list(key, "int")

		__value = self.CLIENT.get(key)

		# set return value to default value.
		if not __value and Conf.SCHEMA.has_key(key) :
			return Conf.SCHEMA[key]["default"]

		return self.get_value_from_value(__value)

	def get_value_from_value (self, GConfValue) :
		if not isinstance(GConfValue, gconf.Value) :
			#raise ValueError, \
			#	"GConfValue must be instance of gconf.Value"
			return None

		__type = GConfValue.type

		if __type == gconf.VALUE_INT :
			__v = GConfValue.get_int()
		elif __type == gconf.VALUE_BOOL :
			__v = GConfValue.get_bool()
		elif __type == gconf.VALUE_STRING :
			__v = GConfValue.get_string()
		elif __type == gconf.VALUE_FLOAT :
			__v = GConfValue.get_float()
		else :
			__v = None

		return __v

	def set_value_by_conf (self, conf_key, value) :
		if not Conf.SCHEMA.has_key(conf_key) :
			return False

		return self.set_value( \
			Conf.SCHEMA[conf_key]["type"], \
			Conf.SCHEMA[conf_key]["key"], \
			value \
		)

	def set_value (self, typeValue, key, value, set_force=False) :

		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			if set_force :
				if not self.create_new(os.path.dirname(key)) :
					return False
			else :
				return False

		if typeValue == "int" :
			self.CLIENT.set_int(key, value)
		elif typeValue == "bool" :
			self.CLIENT.set_bool(key, value)
		elif typeValue == "float" :
			self.CLIENT.set_float(key, value)
		elif typeValue == "string" :
			self.CLIENT.set_string(key, value)
		elif typeValue == "int_list" :
			return self.set_list("int", key, value, set_force)
		else :
			return False

		return True

	def get_list (self, typeValue, key) :

		if typeValue == "int" :
			__type = gconf.VALUE_INT
		elif typeValue == "bool" :
			__type = gconf.VALUE_BOOL
		elif typeValue == "string" :
			__type = gconf.VALUE_STRING
		else :
			return False

		return self.CLIENT.get_list(key, __type)

	def set_list (self, typeValue, key, list_value, set_force=False) :
		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			if set_force :
				if not self.create_new(os.path.dirname(key)) :
					return False
			else :
				return False

		if type(list_value) is not (list or tuple) :
			return False

		if typeValue == "int" :
			__type = gconf.VALUE_INT
		elif typeValue == "bool" :
			__type = gconf.VALUE_BOOL
		elif typeValue == "string" :
			__type = gconf.VALUE_STRING
		else :
			return False

		return self.CLIENT.set_list(key, __type, list_value)

class Notify :

	NOTIFY_ID = dict()
	GCONF_CLIENT = None
	STATE = None
	CALLBACK_FUNCTIONS = dict()

	def __init__ (self, **kwargs) :

		if not kwargs.has_key("state") :
			raise ValueError, "state must be included."
		else :
			self.STATE = kwargs["state"]

		if kwargs.has_key("callback_functions") :
			self.CALLBACK_FUNCTIONS = kwargs["callback_functions"]

		self.GCONF_CLIENT = Client()
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

	def add (self) :

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["show-menubar"]["key"], self.on_toggle_menubar)
		self.STATE.set_value("gconf-notify-cnxid-menubar", notify_id)
		if self.STATE.DATA["toggled-menubar"] :
			self.GCONF_CLIENT.CLIENT.notify_remove(notify_id)

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["show-toolbar"]["key"], self.on_toggle_toolbar)
		self.STATE.set_value("gconf-notify-cnxid-toolbar", notify_id)
		if self.STATE.DATA["toggled-toolbar"] :
			self.GCONF_CLIENT.CLIENT.notify_remove(notify_id)

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["show-bookmark"]["key"], self.on_toggle_bookmark)
		self.STATE.set_value("gconf-notify-cnxid-bookmark", notify_id)
		if self.STATE.DATA["toggled-bookmark"] :
			self.GCONF_CLIENT.CLIENT.notify_remove(notify_id)

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["show-statusbar"]["key"], self.on_toggle_statusbar)
		self.STATE.set_value("gconf-notify-cnxid-statusbar", notify_id)
		if self.STATE.DATA["toggled-statusbar"] :
			self.GCONF_CLIENT.CLIENT.notify_remove(notify_id)

		# toggle toolbar buttons.
		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["use-vim"]["key"], self.on_toggle_vim_button)

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["use-gvim"]["key"], self.on_toggle_gvim_button)

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["use-terminal"]["key"], self.on_toggle_terminal_button)

		# insert new tab.
		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["filename-to-insert-as-new-tab-in-vim"]["key"], \
			self.on_insert_vim \
		)
		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["filename-to-insert-as-new-tab-in-gvim"]["key"], \
			self.on_insert_gvim \
		)

		notify_id = self.GCONF_CLIENT.CLIENT.notify_add( \
			Conf.SCHEMA["tab-position"]["key"], \
			self.on_change_tab_position \
		)

	def on_change_tab_position (self, client, parm1, parm2, parm3) :
		return

		__data = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if __data == "Top" :
			POSITION = gtk.POS_TOP
		elif __data == "Bottom" :
			POSITION = gtk.POS_BOTTOM
		elif __data == "Left" :
			POSITION = gtk.POS_LEFT
		elif __data == "Right" :
			POSITION = gtk.POS_RIGHT
		else :
			POSITION = gtk.POS_BOTTOM

		__tab_pos_old = self.STATE.DATA["notebook"].get_property("tab-pos")

		self.STATE.DATA["notebook"].set_property("tab-pos", POSITION)

	def on_insert_gvim (self, client, parm1, entry, parm3) :
		__data = self.GCONF_CLIENT.get_value_from_value(entry.get_value())

		(__uid, __filenames) = func.parse_argument_of_veet(__data)

		if __uid is not None :
			if self.STATE.DATA["uid"] != __uid : return

		__callback = self.CALLBACK_FUNCTIONS["addPage"]
		for i in __filenames :
			__callback("gvim", filename=i)

	def on_insert_vim (self, client, parm1, entry, parm3) :
		__data = self.GCONF_CLIENT.get_value_from_value(entry.get_value())

		(__uid, __filenames) = func.parse_argument_of_veet(__data)

		if not __uid : return
		try :
			__uid = int(__uid)
		except ValueError :
			return

		if self.STATE.get_value("uid") != __uid : return

		__callback = self.CALLBACK_FUNCTIONS["addPage"]
		for i in __filenames :
			__callback("vim", filename=i)

	def on_toggle_vim_button (self, client, parm1, parm2, parm3) :
		__data = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if not __data :
			self.STATE.DATA["vim-button"].hide()
		else :
			self.STATE.DATA["vim-button"].show()

	def on_toggle_gvim_button (self, client, parm1, parm2, parm3) :
		__data = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if not __data :
			self.STATE.DATA["gvim-button"].hide()
		else :
			self.STATE.DATA["gvim-button"].show()

	def on_toggle_terminal_button (self, client, parm1, parm2, parm3) :
		__data = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if not __data :
			self.STATE.DATA["terminal-button"].hide()
		else :
			self.STATE.DATA["terminal-button"].show()

	def on_toggle_menubar (self, client, parm1, parm2, parm3) :
		__is_activate = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if __is_activate :
			self.STATE.DATA["menubar"].show()
		else :
			self.STATE.DATA["menubar"].hide()

	def on_toggle_toolbar (self, client, parm1, parm2, parm3) :
		__is_activate = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if __is_activate :
			self.STATE.DATA["toolbar"].show()
		else :
			self.STATE.DATA["toolbar"].hide()

	def on_toggle_statusbar (self, client, parm1, parm2, parm3) :
		__is_activate = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if __is_activate :
			self.STATE.DATA["statusbar"].INSTANCE_STATUSBAR.show()
		else :
			self.STATE.DATA["statusbar"].INSTANCE_STATUSBAR.hide()

	def on_toggle_bookmark (self, client, parm1, parm2, parm3) :
		__is_activate = self.GCONF_CLIENT.get_value_from_value(parm2.get_value())

		if __is_activate :
			self.STATE.DATA["panel-bookmark"].show()
		else :
			self.STATE.DATA["panel-bookmark"].hide()

def verify (forceInstall=False) :
	"""
	Check whether key and it's type is correct or not, if not insert
	new schema.
	"""

	if forceInstall :
		install_schema()
	else :
		try :
			GCONF_CLIENT = Client()
			GCONF_CLIENT.set_path(Conf.LOCAL_ROOT)
		except :
			install_schema()
		else :
			if not isinstance(GCONF_CLIENT.CLIENT.get_schema( \
					Conf.SCHEMA["external-command-list"]["schema"]), gconf.Schema) :
				install_schema()

def install_schema () :

	print ">> Installing Schema file, %s" % Conf.SCHEMA_FILE
	a, b, c = \
		os.popen3("gconftool-2 --install-schema-file=%s" % \
			Conf.SCHEMA_FILE)

	# some weird problems, without b.readlines(), c.readlines() hangs up.
	b.readlines()

	if len(c.readlines()) > 0 :
		print ">> Failed to install Schema file."
		return False

	print ">> Install Schema file."
	print ""

	return True

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "GConf.py"

