# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, glob, sre
import gtk
import pango, gnome
import gnome.vfs

import GConf

def get_pixel_size (fontname_full) :
	"""
	Get the pixel height approximately with font description.
	"""
	fd = pango.FontDescription(fontname_full)

	widget =  gtk.Label()
	context = widget.get_pango_context()
	metrics = context.get_metrics(fd)
	__cw = metrics.get_approximate_char_width()
	height = pango.PIXELS(__cw * 5)

	return height

class SYSTEM :
	GCONF_CLIENT = None

	PREFIX = str()
	DIR_LIB = str()
	DIR_INCLUDE = str()
	DIR_SHARE = str()

	def __init__ (self) :

		# set default system directories.
		self.PREFIX = self.get_prefix()
		self.DIR_LIB = "%s/lib" % self.PREFIX
		self.DIR_INCLUDE = "%s/include" % self.PREFIX
		self.DIR_SHARE = "%s/share" % self.PREFIX

	def get_prefix (self) :
		i, o, e = os.popen3("pkg-config  --variable=prefix libgnome-2.0")
		if len(e.readlines()) > 0 :
			raise RuntimeError, "pkg-config must be needed."

		__data = o.readlines()

		if len(__data) < 1 : return None

		return __data[0].strip()

class MIME :

	def get_type (self, filename) :
		try :
			return gnome.vfs.get_mime_type(filename)
		except RuntimeError :
			return None

class Theme :

	SYSTEM = None
	MIME = None

	__STR_DIR_ICON_DEFAULT= "%s/share/icons/gnome"
	__STR_DIR_ICON_SYSTEM0= "%s/share/icons/%s"
	__STR_DIR_ICON_SYSTEM1 = "%s/share/themes/%s/icons"
	__STR_DIR_ICON_HOME = "%s/.themes/%s/icons"

	__STR_DIR_THEME_SYSTEM = "%s/share/themes/%s"
	__STR_DIR_THEME_HOME = "%s/.themes/%s"

	CURRENT = None
	DIR_THEME = str()
	DIR_ICON = str()

	GCONF_KEY_THEME = "/desktop/gnome/interface/icon_theme"

	SRE_ICON_DIRECTORY = sre.compile("[0-9][0-9]*x[0-9][0-9]*$")

	ICON_SIZE_USABLE = list()
	STR_FILE_MIME_TYPE = "gnome-mime-%s.png"

	def __init__ (self) :
		self.GCONF_CLIENT = GConf.Client()

		self.MIME = MIME()
		self.SYSTEM = SYSTEM()

		self.get_directory()

	def get_current (self) :
		name = self.GCONF_CLIENT.get_value(self.GCONF_KEY_THEME)

		if not name :
			raise RuntimeError, "GNOME system configuration error."
		else :
			self.CURRENT = name

		return self.CURRENT

	def get_directory (self) :
		"""
		Get and set the icon directory and theme directory.
		"""

		if not self.CURRENT : self.get_current()

		# get icon directory.
		__DIR_ICON_DEFAULT= self.__STR_DIR_ICON_DEFAULT % \
			(self.SYSTEM.PREFIX)
		__DIR_ICON_SYSTEM0= self.__STR_DIR_ICON_SYSTEM0 % \
			(self.SYSTEM.PREFIX, self.CURRENT)
		__DIR_ICON_SYSTEM1 = self.__STR_DIR_ICON_SYSTEM1 % \
			(self.SYSTEM.PREFIX, self.CURRENT)
		__DIR_ICON_HOME = self.__STR_DIR_ICON_HOME % \
			(os.getenv("HOME"), self.CURRENT)

		# get theme directory
		__DIR_THEME_SYSTEM = self.__STR_DIR_THEME_SYSTEM % \
			(self.SYSTEM.PREFIX, self.CURRENT)
		__DIR_THEME_HOME = self.__STR_DIR_THEME_HOME % \
			(os.getenv("HOME"), self.CURRENT)

		if os.path.exists(__DIR_ICON_HOME) :
			dir_icon = __DIR_ICON_HOME
		elif os.path.exists(__DIR_ICON_SYSTEM0) :
			dir_icon = __DIR_ICON_SYSTEM0
		elif os.path.exists(__DIR_ICON_SYSTEM1) :
			dir_icon = __DIR_ICON_SYSTEM1

		if dir_icon : self.DIR_ICON = dir_icon

		if os.path.exists(__DIR_THEME_HOME) :
			dir_theme = __DIR_THEME_HOME
		elif os.path.exists(__DIR_THEME_SYSTEM) :
			dir_theme = __DIR_THEME_SYSTEM

		if dir_theme : self.DIR_THEME = dir_theme

		return (self.DIR_THEME, self.DIR_ICON)

	def get_usable_icon_size (self) :

		list_icon_size = list()
		for i in glob.glob("%s/*" % self.DIR_ICON) :
			if self.SRE_ICON_DIRECTORY.findall(i) :
				if os.path.exists("%s/mimetypes" % i) :
					list_icon_size.append(os.path.basename(i).split("x")[0])

		self.ICON_SIZE_USABLE = list_icon_size

		return self.ICON_SIZE_USABLE

	def get_icon_size (self, pixel_size) :

		if len(self.ICON_SIZE_USABLE) < 1 :
			self.get_usable_icon_size()

		if len(self.ICON_SIZE_USABLE) < 1 :
			return None

		m = [pixel_size - int(i) for i in self.ICON_SIZE_USABLE]
		n = m[0]
		for i in m[1:] :
			if i < 0 : l = (i * -1)
			else : l = i

			if l < n : n = i

		return self.ICON_SIZE_USABLE[m.index(n)]

	def get_icon_files (self, filename) :

		# get icon info in this system.
		self.get_usable_icon_size()

		__mimetype = self.MIME.get_type(filename)
		if not __mimetype : return dict()

		__filename = self.STR_FILE_MIME_TYPE % \
			("-".join(__mimetype.split("/")))

		list_filename = dict()
		for i in self.ICON_SIZE_USABLE :
			__filename0 = "%s/%sx%s/mimetypes/%s" % \
				(self.DIR_ICON, i, i, __filename)

			if not os.path.exists(__filename0) : continue

			list_filename[i] = __filename0

		return list_filename

	def get_icon_file (self, filename, full_fontname="") :
		__icon_list = self.get_icon_files (filename)

		if len(__icon_list) < 1 : return None

		if not full_fontname :
			__k = __icon_list.keys()
			__k.sort()
			return __icon_list[__k[-1]]
		else :
			pixel_size = get_pixel_size(full_fontname)

			if len(__icon_list) < 1 :
				return None
			else :
				icon_size = self.get_icon_size(pixel_size)
				return __icon_list[icon_size]


if __name__ == "__main__" :
	t = Theme()
	print t.get_icon_file( \
		"/bebop/Downloads/2/vimstudio-0.2.tgz", \
		"Bitstream Vera Sans Mono 220")

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "GNOME.py"


