# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import gtk

import Conf__, Conf, Menu, i18n, func, Window, Handler

_ = i18n.gettext()

COLUMN_ICON = 0
COLUMN_LABEL = 1
COLUMN_UID = 2

class Bookmarks :

	XML = None
	WIDGET = None
	MODEL = None
	BOOKMARK = None

	CALLBACK_FUNCTION = dict()
	STATE = None

	ICON_CATEGORY = None
	ICON_TERMINAL = None
	ICON_VIM = None
	ICON_GVIM = None

	def __init__ (self, xml, **kwargs) :

		if not kwargs.has_key("callback_functions") :
			raise RuntimeError, "callback_functions argument must be give."
		else :
			self.CALLBACK_FUNCTION = kwargs["callback_functions"]

		if not kwargs.has_key("state") :
			raise RuntimeError, "state must be give."
		else :
			self.STATE = kwargs["state"]
			self.BOOKMARKS = self.STATE.get_value("bookmark")

		self.XML = xml
		self.WIDGET = self.XML.get_widget("treeview2")

		# connect handler.
		for __signal, __handler in Handler.PANEL_BOOKMARKS.items() :
			self.WIDGET.connect(__signal, eval(__handler))

		self.__create_icons()
		self.__create_frame()
		self.__insert_data()

		# activate popup menu
		self.WIDGET.connect( \
			"button-release-event", \
			Menu.on_show_popup_bookmarks, \
			self.STATE \
		)

		###############################################################
		# ACTIVATE dnd.
		self.WIDGET.enable_model_drag_dest(Conf.DND_TARGET_TABLE, gtk.gdk.ACTION_DEFAULT)

		self.WIDGET.connect( \
			"drag_data_received", self.on_drag_received_add_item)

	def on_drag_received_add_item \
			(self, widget, context, x, y, selection, targetType, time) :

		__file_path = func.get_filename_from_uri(selection.data)
		__path = self.WIDGET.get_dest_row_at_pos(x, y)

		#Menu.BookmarkItemType(self.BOOKMARKS, self.WIDGET, self.TREESTORE, __path[0], __file_path)

	def __create_frame (self) :

		# create a CellRenderers to render the data
		self.cellpb = gtk.CellRendererPixbuf()
		self.cell = gtk.CellRendererText()
		self.cell1 = gtk.CellRendererText()

		self.tvcolumn = gtk.TreeViewColumn(_("Bookmarks"))
		self.tvcolumn1 = gtk.TreeViewColumn('')
		self.tvcolumn1.set_max_width(1)
		self.tvcolumn.set_expand(True)
		self.WIDGET.set_headers_visible(False)

		self.cell.connect("edited", self.on_rename_edited)

		self.tvcolumn.set_sort_column_id(COLUMN_LABEL)

		self.TREESTORE = gtk.TreeStore(str, type(self.ICON_CATEGORY), str)
		self.WIDGET.set_model(self.TREESTORE)

		# add columns to treeview
		self.WIDGET.append_column(self.tvcolumn)
		self.WIDGET.append_column(self.tvcolumn1)

		# add the cells to the columns - 2 in the first
		self.tvcolumn.pack_start(self.cellpb, False)
		self.tvcolumn.pack_start(self.cell, True)
		self.tvcolumn1.pack_start(self.cell1, True)

		self.tvcolumn.set_cell_data_func( \
			self.cellpb, self.__make_stock_icon)

		self.tvcolumn.set_attributes(self.cell, text=0)
		self.tvcolumn1.set_attributes(self.cell1, text=2, \
			cell_background_set=3)

		# make treeview searchable
		self.WIDGET.set_search_column(COLUMN_LABEL)
		self.WIDGET.set_reorderable(True)

		self.STATE.set_value("bookmark-treestore", self.TREESTORE)

	def __insert_data (self) :
		self.TREESTORE.clear()

		# get bookmarks data.
		__root = self.BOOKMARKS.get_root()

		# insert bookmarks data into treeview.
		self.__insert_item(None, __root)

	def __insert_item (self, parent, uid_list) :
		for i in uid_list :
			__data = self.BOOKMARKS.get_value(i, "asDict")

			if not __data["type"] : continue

			if not __data["label"] : __label = __data["value"]
			__iter_parent = \
				self.add_item(parent, i, __data["type"], __data["label"])

			__child_list = self.BOOKMARKS.get_child(i)
			if len(__child_list) > 0 :
				self.__insert_item(__iter_parent, __child_list)

	def add_item (self, parent, uid, type, label) :

		# set type
		if type == "category" :
			icon = self.ICON_CATEGORY
		elif type == "gvim" :
			icon = self.ICON_GVIM
		elif type == "vim" :
			icon = self.ICON_VIM
		elif type == "terminal" :
			icon = self.ICON_TERMINAL
		elif type == "external_command" :
			icon = self.ICON_EXTERNAL_COMMAND
		else :
			icon = None

		child = self.TREESTORE.append(parent, [label, icon, uid])

		return child

	def __get_iter (self) :
		__tree_selection = self.WIDGET.get_selection()

		(__tree_store, __tree_iter) = __tree_selection.get_selected()

		return __tree_iter

	def get_value (self, column_number) :
		__tree_selection = self.WIDGET.get_selection()
		(__tree_store, __tree_iter) = __tree_selection.get_selected()

		__value = self.TREESTORE.get_value(__tree_iter, column_number)
		return __value

	def __get_path (self) :
		__tree_selection = self.WIDGET.get_selection()

		(__tree_store, __tree_iter) = __tree_selection.get_selected()

		__path = self.TREESTORE.get_path(__tree_iter)

		return __path

	##########################################GDK_BUTTON_PRESS#########################
	# HANDLER
	def on_bookmark_row_expanded \
			(self, treeview, iter, path, user_param1=None) :
		return

	def on_bookmark_row_collapsed \
			(self, treeview, iter, path, user_param1=None) :
		return

	def on_bookmark_row_activated \
			(self, treeview, path, view_column, user_param1=None) :

		uid = self.get_value(COLUMN_UID)
		__data = self.BOOKMARKS.get_value(uid, "asDict")

		if __data["type"] == "category" :
			if self.WIDGET.row_expanded(path) :
				self.WIDGET.collapse_row(path)
			else :
				self.WIDGET.expand_row(path, False)

			return

		try :
			__setting = self.BOOKMARKS.get_setting(uid)
		except IndexError :
			__setting = None

		# run command.
		__func = self.CALLBACK_FUNCTION["addPage"]

		if __data["type"] == "terminal" :
			__func(__data["type"], directory=__data["value"], setting=__setting)
		elif __data["type"] == "external_command" :
			__func(__data["type"], labelname=__data["label"], \
				external_command=__data["value"], setting=__setting)
		elif __data["type"] in ("vim", "gvim") :
			__func(__data["type"], filename=__data["value"], setting=__setting)

		self.STATE.get_value("panel-bookmark").hide()

		return

	def on_rename_edited (self, cell, path, new_label) :
		__path = tuple([ int(i) for i in path.split(":")])

		cell.set_property("editable", False)
		self.rename_by_path(__path, new_label)

		return

	def rename_by_path (self, path, s) :
		#if not self.BOOKMARKS.edit(path, label=s) :
		#	return gtk.FALSE
		__uid = self.get_value(COLUMN_UID)
		self.BOOKMARKS.set_value(__uid, "label", s)

		self.TREESTORE[path][0] = s

		return gtk.TRUE

	####################################################################
	# misc
	def __create_icons (self) :
		self.ICON_CATEGORY = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "category.png"))
		self.ICON_EXTERNAL_COMMAND = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "external-command.png"))
		self.ICON_TERMINAL = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "terminal.png"))
		self.ICON_VIM = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "vim.png"))
		self.ICON_GVIM = func.create_icon(os.path.join(Conf__.DIR_PIXMAPS, "gvim.png"))

	def __make_stock_icon (self, tvcolumn, cell, model, iter):
		icon = model.get_value(iter, 1)
		cell.set_property('pixbuf', icon)

		return

"""
Description__create_data
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"


