# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os

import i18n

_ = i18n.gettext()

import gtk, vte
import Label, GConf, Conf, child

class One :

	NOTEBOOK = None
	STATE = None

	WIDGET_ROOT = None
	LIST_TAB = list()

	INSTANCE_LABEL = None
	INSTANCE_SCROLLEDWINDOW = None

	STR_TAB_NAME = "None"

	WITHOUT_CLOSEBUTTON = False

	def __init__ (self, **kwargs) :

		if not kwargs.has_key("state") :
			raise ValueError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			if not self.STATE.DATA.has_key("notebook") :
				raise ValueError, _("notebook was not instantiated.")
			else :
				self.NOTEBOOK = self.STATE.DATA["notebook"]

		if kwargs.has_key("without_closebutton") :
			self.WITHOUT_CLOSEBUTTON = kwargs["without_closebutton"]

		self.WIDGET_ROOT = self.NOTEBOOK.get_root_window()
		self.LIST_TAB = self.STATE.DATA["tab-list"]

	def __del__ (self) :
		del(self.Child)
		try :
			del(self.INSTANCE_SCROLLEDWINDOW)
		except AttributeError :
			pass

		del(self.INSTANCE_LABEL)

	def on_signal_remove_tab (self, widget) :

		self.Child.exit()

		currnet_page = self.NOTEBOOK.get_current_page()
		self.NOTEBOOK.remove_page(currnet_page)
		self.LIST_TAB.remove(currnet_page)

		try :
			self.NOTEBOOK.set_current_page(self.STATE.get_value("tab-history").get_previous())
		except IndexError :
			pass

	def addLabel (self, label) :

		if not isinstance(label.WIDGET, gtk.Widget) :
			raise ValueError, \
				_("label must be the subclass of gtk.Widget.")

		self.INSTANCE_LABEL = label

	def __makeLabel (self) :

		if not self.INSTANCE_LABEL :
			self.INSTANCE_LABEL = Label.TabWithClose( \
				state=self.STATE, \
				labelname=self.STR_TAB_NAME, \
				without_closebutton=self.WITHOUT_CLOSEBUTTON)

	#########################################################
	# Child
	def connectSignalChild (self) :
		self.Child.WIDGET.connect(self.Child.SIGNAL_REMOVE, \
			self.on_signal_remove_tab)

	def addChild (self, child) :

		self.Child = child

		self.connectSignalChild()

		self.__makeLabel()

		# create scrolled window
		self.INSTANCE_SCROLLEDWINDOW = gtk.ScrolledWindow()
		self.INSTANCE_SCROLLEDWINDOW.set_policy( \
			gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)

		self.INSTANCE_SCROLLEDWINDOW.set_border_width(0)
		self.INSTANCE_SCROLLEDWINDOW.set_shadow_type(gtk.SHADOW_NONE)
		self.INSTANCE_SCROLLEDWINDOW.set_property("can-focus", True)
		self.INSTANCE_SCROLLEDWINDOW.set_property("has-focus", True)
		self.INSTANCE_SCROLLEDWINDOW.set_property("sensitive", True)

		self.INSTANCE_SCROLLEDWINDOW.grab_focus()

		self.INSTANCE_SCROLLEDWINDOW.add_with_viewport(self.Child.WIDGET)
		self.INSTANCE_SCROLLEDWINDOW.show()

		# add ScrolledWindow in notebook
		self.INSTANCE_LABEL.registerChild(self.Child)
		self.NOTEBOOK.append_page( \
			self.INSTANCE_SCROLLEDWINDOW, self.INSTANCE_LABEL.WIDGET)

		# focus in widget.
		self.NOTEBOOK.set_current_page(self.NOTEBOOK.get_n_pages() - 1)

	def addVTE (self, instancevte) :

		if not isinstance(instancevte.WIDGET, vte.Terminal) :
			raise ValueError, _("vte must be instance of VTE class.")

		self.Child = instancevte

		self.connectSignalChild()

		self.__makeLabel()

		# create scrolled window
		self.INSTANCE_SCROLLEDWINDOW = gtk.ScrolledWindow()
		self.INSTANCE_SCROLLEDWINDOW.set_policy( \
			gtk.POLICY_NEVER, gtk.POLICY_NEVER)

		self.INSTANCE_SCROLLEDWINDOW.set_border_width(0)
		self.INSTANCE_SCROLLEDWINDOW.set_shadow_type(gtk.SHADOW_NONE)
		self.INSTANCE_SCROLLEDWINDOW.set_property("can-focus", True)
		self.INSTANCE_SCROLLEDWINDOW.set_property("has-focus", True)

		self.INSTANCE_SCROLLEDWINDOW.add_with_viewport(self.Child.WIDGET)
		self.INSTANCE_SCROLLEDWINDOW.show()

		# add ScrolledWindow in notebook
		self.INSTANCE_LABEL.registerChild(self.Child)
		self.NOTEBOOK.append_page( \
			self.INSTANCE_SCROLLEDWINDOW, self.INSTANCE_LABEL.WIDGET)

		# focus in VTE.
		self.NOTEBOOK.set_current_page(self.NOTEBOOK.get_n_pages() - 1)
		self.Child.WIDGET.grab_focus()

	def addWidget (self, widget, labelname=None) :

		self.Child = widget

		if not isinstance(widget, gtk.Widget) :
			raise ValueError, \
				_("widget must be instance of gtk.Widget class.")

		self.INSTANCE_LABEL = Label.TabWithClose( \
			state=self.STATE, filename=labelname, \
			without_closebutton=self.WITHOUT_CLOSEBUTTON)

		self.NOTEBOOK.append_page( \
			widget, self.INSTANCE_LABEL.WIDGET)

		self.NOTEBOOK.set_current_page(self.NOTEBOOK.get_n_pages() - 1)

		self.Child.grab_focus()

class List :

	LIST = list()

	def __init__ (self) :
		pass

	def __len__ (self) :
		return len(self.LIST)

	def __iter__ (self) :
		return self.LIST.__iter__()

	def add (self, tab) :
		self.LIST.append(tab)

	def append (self, tab) :
		"""
		depreciated.
		"""
		self.add(tab)

	def remove (self, index) :
		try :
			del(self.LIST[index])
		except IndexError :
			raise IndexError, "%d does not exists in tab-list." % index

	def get (self, index) :
		try :
			return self.LIST[index]
		except IndexError :
			raise IndexError, "%d does not exists in tab-list." % index

	def get_child (self, index) :
		__one = self.get(index)

		return __one.Child

	def get_child_widget (self, index) :
		__child = self.get_child(index)

		return __child.WIDGET

class History :
	LIST = list()

	def add (self, page_number) :
		self.LIST.append(page_number)
		if len(self.LIST) > 3 :
			self.LIST.reverse()
			self.LIST.pop()
			self.LIST.reverse()

	def get_previous (self) :
		return self.LIST[0]

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "TAB.py"

